/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/get_monitoring_ips.hh"

#include "src/cfg.hh"

#include "liblog/liblog.hh"

#include <fstream>

namespace Fred {
namespace Logger {

namespace {

std::vector<boost::asio::ip::address> get_monitoring_ips_from_file(const std::string& _filename)
{
    LIBLOG_DEBUG("get_monitoring_ips_from_file: {}", _filename);
    std::vector<boost::asio::ip::address> monitoring_ips;

    std::ifstream infile{_filename};

    std::string ipstr;
    while (infile >> ipstr)
    {

        boost::system::error_code boost_error_code;
        const auto ip = boost::asio::ip::address::from_string(ipstr, boost_error_code);
        if (boost_error_code)
        {
            LIBLOG_WARNING("INVALID monitoring ip: {}", ipstr);
        }

        monitoring_ips.emplace_back(ip);
        LIBLOG_DEBUG("monitoring ip: {} (imported as {})", ipstr, ip.to_string());
    }
    return monitoring_ips;
}

} // namespace Fred::Logger::{anonymous}

std::vector<boost::asio::ip::address> get_monitoring_ips()
{
    static const std::vector<boost::asio::ip::address> monitoring_ips{get_monitoring_ips_from_file(Cfg::Options::get().logger.monitoring_hosts_file)};
    return monitoring_ips;
}

} // namespace Fred::Logger
} // namespace Fred
