/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "test/fixture.hh"

#include "libpg/query.hh"

namespace Test {
namespace Fixture {

namespace {

Fred::Logger::LogEntry::LogEntryId insert_log_entry(
        const LibPg::PgRwTransaction& _rw_transaction,
        const Fred::Logger::LogEntry::TimeBegin& _time_begin,
        const Fred::Logger::LogEntry::ServiceName& _service_name)
{
    using QueryArguments = LibPg::FixedSizeQueryArguments<2>;

    LibPg::exec(_rw_transaction,
         "INSERT INTO request (time_begin, service_id) "
         "VALUES (" + QueryArguments::parameter<0>().as_timestamp() + ", "
                 "(SELECT id FROM service WHERE name = " + QueryArguments::parameter<1>().as_text() + "))",
         LibPg::make_fixed_size_query_arguments(_time_begin.get(), _service_name.get()));
         // clang-format on

    const LibPg::PgResultTuples dbres = exec(_rw_transaction, "SELECT currval('request_id_seq'::REGCLASS) AS id");

    const auto log_entry_id = Fred::Logger::LogEntry::LogEntryId{dbres[LibPg::RowIndex{0}][LibPg::ColumnIndex{0}].as<unsigned long long>()};

    return log_entry_id;
}

Fred::Logger::LogEntry::LogEntryId insert_log_entry(
        const LibPg::PgRwTransaction& _rw_transaction,
        const Fred::Logger::LogEntry::TimeBegin& _time_begin,
        const Fred::Logger::LogEntry::ServiceName& _service_name,
        const Fred::Logger::LogEntry::LogEntryTypeName& _log_entry_type,
        const Fred::Logger::LogEntry::UserName& _username,
        const Fred::Logger::LogEntry::UserId& _user_id,
        const Fred::Logger::LogEntry::LogEntryObjectTypeName& _object_type,
        const Fred::Logger::LogEntry::ObjectId& _object_id,
        const Fred::Logger::LogEntry::ObjectIdent& _object_ident)
{

    using QueryArguments = LibPg::FixedSizeQueryArguments<5>;

    LibPg::exec(
            _rw_transaction,
            // clang-format off
            "INSERT INTO request (time_begin, service_id, request_type_id, user_name, user_id) "
            "VALUES (" + QueryArguments::parameter<0>().as_timestamp() + ", "
                    "(SELECT id FROM service WHERE name = " + QueryArguments::parameter<1>().as_text() + "), " +
                    QueryArguments::parameter<2>().as_big_int() + ", " +
                    QueryArguments::parameter<3>().as_text() + ", " +
                    QueryArguments::parameter<4>().as_big_int() + ")",
            LibPg::make_fixed_size_query_arguments(
                    _time_begin.get(),
                    _service_name.get(),
                    _log_entry_type.get(),
                    _username.get(),
                    _user_id.get()));
            // clang-format on

    const LibPg::PgResultTuples dbres = exec(_rw_transaction, "SELECT currval('request_id_seq'::REGCLASS) AS id");

    const auto log_entry_id = Fred::Logger::LogEntry::LogEntryId{dbres[LibPg::RowIndex{0}][LibPg::ColumnIndex{0}].as<unsigned long long>()};

    {
    using QueryArguments = LibPg::FixedSizeQueryArguments<6>;

    LibPg::exec(
            _rw_transaction,
            // clang-format off
            "INSERT INTO request_object_ref (request_id, request_time_begin, request_service_id, object_type_id, object_id, object_ident) "
            "SELECT " + QueryArguments::parameter<0>().as_big_int() + ", " +
                        QueryArguments::parameter<1>().as_timestamp() + ", "
                        "(SELECT id FROM service WHERE name = " + QueryArguments::parameter<2>().as_text() + "), "
                        "id, " +
                        QueryArguments::parameter<4>().as_big_int() + ", " +
                        QueryArguments::parameter<5>().as_text() + " "
              "FROM request_object_type "
             "WHERE name = " + QueryArguments::parameter<3>().as_text(),
            // clang-format on
            LibPg::make_fixed_size_query_arguments(
                    log_entry_id,
                    _time_begin,
                    _service_name,
                    _object_type,
                    _object_id,
                    _object_ident));
    }

    return log_entry_id;
}

} // namespace Test::Fixture::{anonymous}

// empty

Empty::Empty(const LibPg::PgRwTransaction& _rw_transaction)
{
}

// create_session

HasEmptyCreateSessionData::HasEmptyCreateSessionData(const LibPg::PgRwTransaction& _rw_transaction)
{
}

HasAllCreateSessionData::HasAllCreateSessionData(const LibPg::PgRwTransaction& _rw_transaction)
    : create_session_data{
              ::Fred::Logger::LogEntry::UserName{"username"},
              ::Fred::Logger::LogEntry::UserId{1}}
{
}

HasRequiredCreateSessionData::HasRequiredCreateSessionData(const LibPg::PgRwTransaction& _rw_transaction)
    : create_session_data{
              ::Fred::Logger::LogEntry::UserName{"username"},
              boost::none}
{
}

// close_session

HasEmptyCloseSessionData::HasEmptyCloseSessionData(const LibPg::PgRwTransaction& _rw_transaction)
{
}

HasCloseSessionDataEmptyDb::HasCloseSessionDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction)
    : close_session_data{
              ::Fred::Logger::SessionIdent{
                      ::Fred::Logger::LogEntry::SessionId{1},
                      ::Fred::Logger::LogEntry::SessionLoginDate{
                              boost::posix_time::from_iso_string("19700101T000000.000")}}}
{
    LibPg::exec(_rw_transaction, "DELETE FROM session");
}

HasCloseSessionDataFullDb::HasCloseSessionDataFullDb(const LibPg::PgRwTransaction& _rw_transaction)
    : close_session_data{
              ::Fred::Logger::create_session(
                      _rw_transaction,
                      ::Fred::Logger::LogEntry::UserName{"usernme"},
                      ::Fred::Logger::LogEntry::UserId{1})
                      .session_ident}
{
}

// get_services

HasGetServicesDataEmptyDb::HasGetServicesDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_services_result{}
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
    LibPg::exec(_rw_transaction, "DELETE FROM result_code");
    LibPg::exec(_rw_transaction, "DELETE FROM service");
}

HasGetServicesDataFullDb::HasGetServicesDataFullDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_services_result{
        {::Fred::Logger::LogEntry::ServiceName{"SERVICE"}}}
{
    //const auto result =
    //        ::Fred::Logger::get_services(
    //               _rw_transaction);
    //for (const auto& log_entry_service : result.log_entry_services)
    //{
    //    std::cout << log_entry_service.get() << std::endl;
    //}
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
    LibPg::exec(_rw_transaction, "DELETE FROM result_code");
    LibPg::exec(_rw_transaction, "DELETE FROM service");
    LibPg::exec(
            _rw_transaction,
            "INSERT INTO service (partition_postfix, name) VALUES ('service_', 'SERVICE')");
}

// get_results

HasEmptyGetResultsData::HasEmptyGetResultsData(const LibPg::PgRwTransaction& _rw_transaction)
    : get_results_data{}
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
    LibPg::exec(_rw_transaction, "DELETE FROM result_code");
    LibPg::exec(_rw_transaction, "DELETE FROM service");
    LibPg::exec(
            _rw_transaction,
            "INSERT INTO service (partition_postfix, name) VALUES ('service_', 'SERVICE')");
}

HasGetResultsDataEmptyDb::HasGetResultsDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_results_data{
              ::Fred::Logger::LogEntry::ServiceName{"SERVICE"}}
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
    LibPg::exec(_rw_transaction, "DELETE FROM result_code");
    LibPg::exec(_rw_transaction, "DELETE FROM service");
}

HasGetResultsDataFullDb::HasGetResultsDataFullDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_results_data{
              ::Fred::Logger::LogEntry::ServiceName{"SERVICE"}},
      get_results_result{
              {{"Rc", 0}}}
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
    LibPg::exec(_rw_transaction, "DELETE FROM result_code");
    LibPg::exec(_rw_transaction, "DELETE FROM service");
    LibPg::exec(
            _rw_transaction,
            "INSERT INTO service (partition_postfix, name) VALUES ('service_', 'SERVICE')");
    const auto db_result =
            exec(_rw_transaction,
                    LibPg::make_query() << "SELECT" <<
                    LibPg::item<::Fred::Logger::LogEntry::ServiceId>() << "CURRVAL('service_id_seq'::REGCLASS)");
    LibPg::exec(
            _rw_transaction,
            LibPg::make_query() <<
            "INSERT INTO result_code (service_id, result_code, name) "
            "VALUES (" << LibPg::parameter<::Fred::Logger::LogEntry::ServiceId>().as_integer() << ", 0, 'Rc')",
            db_result.front().get<::Fred::Logger::LogEntry::ServiceId>());
}

// get_log_entry_types

HasEmptyGetLogEntryTypesData::HasEmptyGetLogEntryTypesData(const LibPg::PgRwTransaction& _rw_transaction)
    : get_log_entry_types_data{}
{
}

HasGetLogEntryTypesDataEmptyDb::HasGetLogEntryTypesDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_log_entry_types_data{
              ::Fred::Logger::LogEntry::ServiceName{"SERVICE"}},
      get_log_entry_types_result{}
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
}

HasGetLogEntryTypesDataFullDb::HasGetLogEntryTypesDataFullDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_log_entry_types_data{
              ::Fred::Logger::LogEntry::ServiceName{"SERVICE"}},
      get_log_entry_types_result{
          {::Fred::Logger::LogEntry::LogEntryTypeName{"Rt"}}}
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_type");
    LibPg::exec(_rw_transaction, "DELETE FROM result_code");
    LibPg::exec(_rw_transaction, "DELETE FROM service");
    LibPg::exec(
            _rw_transaction,
            "INSERT INTO service (partition_postfix, name) VALUES ('service_', 'SERVICE')");
    const auto db_result =
            exec(_rw_transaction,
                    LibPg::make_query() << "SELECT" <<
                    LibPg::item<::Fred::Logger::LogEntry::ServiceId>() << "CURRVAL('service_id_seq'::REGCLASS)");
    LibPg::exec(
            _rw_transaction,
            LibPg::make_query() <<
            "INSERT INTO request_type (name, service_id) "
            "VALUES ('Rt', " << LibPg::parameter<::Fred::Logger::LogEntry::ServiceId>().as_integer() << ")",
            db_result.front().get<::Fred::Logger::LogEntry::ServiceId>());
}

// get_object_reference_types

HasGetObjectReferenceTypesDataEmptyDb::HasGetObjectReferenceTypesDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction)
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_object_type");
}

HasGetObjectReferenceTypesDataFullDb::HasGetObjectReferenceTypesDataFullDb(const LibPg::PgRwTransaction& _rw_transaction)
    : get_object_reference_types_result{
        {::Fred::Logger::LogEntry::LogEntryObjectTypeName{"OBJECTTYPE"}}
    }
{
    LibPg::exec(_rw_transaction, "DELETE FROM request_object_type");
    //const auto result =
    //        ::Fred::Logger::get_object_reference_types(
    //               _rw_transaction);
    //for (const auto& object_reference_type : result.object_reference_types)
    //{
    //    std::cout << object_reference_type.get() << std::endl;
    //}
    LibPg::exec(
            _rw_transaction,
            "INSERT INTO request_object_type (name) VALUES ('OBJECTTYPE')");
}

//

HasSeveralLogEntries::HasSeveralLogEntries(const LibPg::PgRwTransaction& _rw_transaction)
{
    Fred::Logger::LogEntry::TimeBegin time_begin{boost::posix_time::second_clock::local_time()};
    Fred::Logger::LogEntry::ServiceName service_name{"EPP"};
    Fred::Logger::LogEntry::LogEntryTypeName log_entry_type{"DomainCheck"}; // FIXME
    const Fred::Logger::LogEntry::UserName username{"test_username"};
    const Fred::Logger::LogEntry::UserId user_id{1};
    const Fred::Logger::LogEntry::LogEntryObjectTypeName object_type{"domain"};
    const Fred::Logger::LogEntry::ObjectId object_id{0};
    const Fred::Logger::LogEntry::ObjectIdent object_ident{"222"};

    for (std::size_t i = 0; i < 12; ++i)
    {
        if (i % 2 == 0)
        {
            service_name = Fred::Logger::LogEntry::ServiceName{"EPP"};
        }
        else
        {
            service_name = Fred::Logger::LogEntry::ServiceName{"WebAdmin"};
            time_begin = Fred::Logger::LogEntry::TimeBegin{time_begin.get() - boost::gregorian::days(1)};
        }

        if (i < 5)
        {
            insert_log_entry(_rw_transaction, time_begin, service_name);
        }
        else
        {
            insert_log_entry(_rw_transaction, time_begin, service_name, log_entry_type, username, user_id, object_type, object_id, object_ident);
        }
    }
}

LogEntry001::LogEntry001(const LibPg::PgRwTransaction& _rw_transaction)
{
    const auto log_entry_id = insert_log_entry(
            _rw_transaction,
            existing_log_entry.time_begin,
            existing_log_entry.service_name,
            existing_log_entry.log_entry_type,
            existing_log_entry.username,
            existing_log_entry.user_id,
            existing_log_entry.object_type,
            existing_log_entry.object_id,
            existing_log_entry.object_ident);
    existing_log_entry.id = log_entry_id;
}

} // namespace Test::Fixture
} // namespace Test
