/*
 * Copyright (C) 2019-2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FIXTURE_HH_1F08E69090D64E25A1F3894EC6F4B8CC
#define FIXTURE_HH_1F08E69090D64E25A1F3894EC6F4B8CC

#include "src/close_log_entry.hh"
#include "src/close_session.hh"
#include "src/create_log_entry.hh"
#include "src/create_session.hh"
#include "src/get_log_entry_info.hh"
#include "src/get_log_entry_types.hh"
#include "src/get_object_reference_types.hh"
#include "src/get_results.hh"
#include "src/get_services.hh"
#include "src/list_log_entries.hh"
#include "src/log_entry.hh"
#include "src/log_entry_ident.hh"

#include "libpg/pg_rw_transaction.hh"

#include <boost/date_time/posix_time/posix_time.hpp>

#include <string>

namespace Test {
namespace Fixture {

using CloseLogEntryData = ::Fred::Logger::CloseLogEntryRequest;
using CloseSessionData = ::Fred::Logger::CloseSessionRequest;
using CreateLogEntryData = ::Fred::Logger::CreateLogEntryRequest;
using CreateSessionData = ::Fred::Logger::CreateSessionRequest;
using GetLogEntryInfoData = ::Fred::Logger::GetLogEntryInfoRequest;
using GetLogEntryTypesData = ::Fred::Logger::GetLogEntryTypesRequest;
using GetObjectReferenceTypesData = ::Fred::Logger::GetObjectReferenceTypesRequest;
using GetResultsData = ::Fred::Logger::GetResultsRequest;
using ListLogEntriesData = ::Fred::Logger::ListLogEntriesRequest;

using CreateLogEntryResult = ::Fred::Logger::CreateLogEntryReply;
using CreateSessionResult = ::Fred::Logger::CreateSessionReply;
using GetLogEntryInfoResult = ::Fred::Logger::GetLogEntryInfoReply;
using GetLogEntryTypesResult = ::Fred::Logger::GetLogEntryTypesReply;
using GetObjectReferenceTypesResult = ::Fred::Logger::GetObjectReferenceTypesReply;
using GetResultsResult = ::Fred::Logger::GetResultsReply;
using ListLogEntriesResult = ::Fred::Logger::ListLogEntriesReply;
using GetServicesResult = ::Fred::Logger::GetServicesReply;

// empty

struct Empty
{
    Empty(const LibPg::PgRwTransaction& _rw_transaction);
};

// create_session

struct HasEmptyCreateSessionData
{
    Fixture::CreateSessionData create_session_data;

    HasEmptyCreateSessionData(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasAllCreateSessionData
{
    Fixture::CreateSessionData create_session_data;

    HasAllCreateSessionData(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasRequiredCreateSessionData
{
    Fixture::CreateSessionData create_session_data;

    HasRequiredCreateSessionData(const LibPg::PgRwTransaction& _rw_transaction);
};

// close_session

struct HasEmptyCloseSessionData
{
    Fixture::CloseSessionData close_session_data;

    HasEmptyCloseSessionData(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasCloseSessionDataEmptyDb
{
    Fixture::CloseSessionData close_session_data;

    HasCloseSessionDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasCloseSessionDataFullDb
{
    Fixture::CloseSessionData close_session_data;

    HasCloseSessionDataFullDb(const LibPg::PgRwTransaction& _rw_transaction);
};

// get_services

struct HasGetServicesDataEmptyDb
{
    Fixture::GetServicesResult get_services_result;

    HasGetServicesDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasGetServicesDataFullDb
{
    Fixture::GetServicesResult get_services_result;

    HasGetServicesDataFullDb(const LibPg::PgRwTransaction& _rw_transaction);
};

// get_results

struct HasEmptyGetResultsData
{
    Fixture::GetResultsData get_results_data;

    HasEmptyGetResultsData(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasGetResultsDataEmptyDb
{
    Fixture::GetResultsData get_results_data;

    HasGetResultsDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasGetResultsDataFullDb
{
    Fixture::GetResultsData get_results_data;
    Fixture::GetResultsResult get_results_result;

    HasGetResultsDataFullDb(const LibPg::PgRwTransaction& _rw_transaction);
};

// get_log_entry_types

struct HasEmptyGetLogEntryTypesData
{
    Fixture::GetLogEntryTypesData get_log_entry_types_data;

    HasEmptyGetLogEntryTypesData(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasGetLogEntryTypesDataEmptyDb
{
    Fixture::GetLogEntryTypesData get_log_entry_types_data;
    Fixture::GetLogEntryTypesResult get_log_entry_types_result;

    HasGetLogEntryTypesDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasGetLogEntryTypesDataFullDb
{
    Fixture::GetLogEntryTypesData get_log_entry_types_data;
    Fixture::GetLogEntryTypesResult get_log_entry_types_result;

    HasGetLogEntryTypesDataFullDb(const LibPg::PgRwTransaction& _rw_transaction);
};

// get_object_reference_types

struct HasGetObjectReferenceTypesDataEmptyDb
{
    HasGetObjectReferenceTypesDataEmptyDb(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasGetObjectReferenceTypesDataFullDb
{
    Fixture::GetObjectReferenceTypesResult get_object_reference_types_result;

    HasGetObjectReferenceTypesDataFullDb(const LibPg::PgRwTransaction& _rw_transaction);
};

//

struct HasSeveralLogEntries
{
    HasSeveralLogEntries(const LibPg::PgRwTransaction& _rw_transaction);
};

struct LogEntry001
{
    struct LogEntry
    {
        Fred::Logger::LogEntry::LogEntryId id;
        const Fred::Logger::LogEntry::TimeBegin time_begin;
        const Fred::Logger::LogEntry::ServiceName service_name;
        const Fred::Logger::LogEntry::LogEntryTypeName log_entry_type;
        const Fred::Logger::LogEntry::UserName username;
        const Fred::Logger::LogEntry::UserId user_id;
        const Fred::Logger::LogEntry::LogEntryObjectTypeName object_type;
        const Fred::Logger::LogEntry::ObjectId object_id;
        const Fred::Logger::LogEntry::ObjectIdent object_ident;
        const Fred::Logger::LogEntry::IsMonitoring monitoring_status;
    };

    LogEntry existing_log_entry{
        Fred::Logger::LogEntry::LogEntryId{1},
        Fred::Logger::LogEntry::TimeBegin{boost::posix_time::second_clock::local_time()},
        Fred::Logger::LogEntry::ServiceName{"EPP"},
        Fred::Logger::LogEntry::LogEntryTypeName{"DomainCheck"},
        Fred::Logger::LogEntry::UserName{"test_username"},
        Fred::Logger::LogEntry::UserId{1},
        Fred::Logger::LogEntry::LogEntryObjectTypeName{"domain"},
        Fred::Logger::LogEntry::ObjectId{0},
        Fred::Logger::LogEntry::ObjectIdent{"222"},
        Fred::Logger::LogEntry::IsMonitoring{false}};

    LogEntry001(const LibPg::PgRwTransaction& _rw_transaction);
};

struct HasLogEntryIdentWithNonexistentService
{
    LogEntry001 log_entry_001;

    HasLogEntryIdentWithNonexistentService(const LibPg::PgRwTransaction& _rw_transaction)
            : log_entry_001{_rw_transaction},
              log_entry_ident_with_nonexistent_service{
                    log_entry_001.existing_log_entry.id,
                    log_entry_001.existing_log_entry.time_begin,
                    Fred::Logger::LogEntry::ServiceName{"nonexistent_ " + log_entry_001.existing_log_entry.service_name.get()},
                    log_entry_001.existing_log_entry.monitoring_status}
    {
    }

    ::Fred::Logger::LogEntryIdent log_entry_ident_with_nonexistent_service;
};

struct HasNonexistentLogEntryIdent
{
    HasNonexistentLogEntryIdent(const LibPg::PgRwTransaction& _rw_transaction)
    {
    }

    ::Fred::Logger::LogEntryIdent nonexistent_log_entry_ident{
        Fred::Logger::LogEntry::LogEntryId{0},
        Fred::Logger::LogEntry::TimeBegin{boost::posix_time::second_clock::local_time()},
        Fred::Logger::LogEntry::ServiceName{"EPP"},
        Fred::Logger::LogEntry::IsMonitoring{true}};
};

struct HasLogEntryIdentWithInvalidTimeBegin
{
    LogEntry001 log_entry_001;

    HasLogEntryIdentWithInvalidTimeBegin(const LibPg::PgRwTransaction& _rw_transaction)
            : log_entry_001(_rw_transaction)
    {
    }

    ::Fred::Logger::LogEntryIdent log_entry_ident_with_invalid_time_begin_not_a_date_time{
            Fred::Logger::LogEntry::LogEntryId{1},
            Fred::Logger::LogEntry::TimeBegin{boost::posix_time::not_a_date_time},
            Fred::Logger::LogEntry::ServiceName{"EPP"},
            Fred::Logger::LogEntry::IsMonitoring{true}};
    ::Fred::Logger::LogEntryIdent log_entry_ident_with_invalid_time_begin_pos_infin{
            Fred::Logger::LogEntry::LogEntryId{1},
            Fred::Logger::LogEntry::TimeBegin{boost::posix_time::pos_infin},
            Fred::Logger::LogEntry::ServiceName{"EPP"},
            Fred::Logger::LogEntry::IsMonitoring{true}};
    ::Fred::Logger::LogEntryIdent log_entry_ident_with_invalid_time_begin_neg_infin{
            Fred::Logger::LogEntry::LogEntryId{1},
            Fred::Logger::LogEntry::TimeBegin{boost::posix_time::neg_infin},
            Fred::Logger::LogEntry::ServiceName{"EPP"},
            Fred::Logger::LogEntry::IsMonitoring{true}};
};

struct HasLogEntryIdent
{
    LogEntry001 log_entry_001;
    ::Fred::Logger::LogEntryIdent log_entry_ident;

    HasLogEntryIdent(const LibPg::PgRwTransaction& _rw_transaction)
            : log_entry_001(_rw_transaction),
              log_entry_ident{
                    log_entry_001.existing_log_entry.id,
                    log_entry_001.existing_log_entry.time_begin,
                    log_entry_001.existing_log_entry.service_name,
                    log_entry_001.existing_log_entry.monitoring_status}
    {
    }
};

struct HasEmptyCreateLogEntryData
{
    Fixture::CreateLogEntryData create_log_entry_data;

    HasEmptyCreateLogEntryData(const LibPg::PgRwTransaction& _rw_transaction)
    {
    }
};

} // namespace Test::Fixture
} // namespace Test

#endif
