/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "test/fixture.hh"
#include "test/util.hh"

#include "src/close_log_entry.hh"
#include "src/exceptions.hh"

#include <boost/test/unit_test.hpp>

namespace Test {

BOOST_AUTO_TEST_SUITE(TestCloseLogEntry)

BOOST_FIXTURE_TEST_CASE(log_entry_does_not_exist, Util::AddFixture<Util::SupplyCtx<Fixture::HasNonexistentLogEntryIdent>>)
{
    Fixture::CloseLogEntryData close_log_entry_data{fixture.nonexistent_log_entry_ident};

    BOOST_CHECK_THROW(::Fred::Logger::close_log_entry(
                    fixture.transaction,
                    close_log_entry_data.log_entry_ident,
                    close_log_entry_data.log_entry_content,
                    close_log_entry_data.log_entry_properties,
                    close_log_entry_data.object_references,
                    close_log_entry_data.result_code_or_name,
                    close_log_entry_data.session_ident
                    ),
            ::Fred::Logger::LogEntryDoesNotExist);
}

BOOST_AUTO_TEST_SUITE_END() // TestCloseLogEntry

} // namespace Test
